from ci.models import CiProject, TestModule, TestRun
from django.conf import settings
from django.core.management.base import BaseCommand
import json
import os


class Command(BaseCommand):
    help = 'loads testresults.json into database'

    def handle(self, name="local_site", branch="master", filename="testresults.json", **options):

        fileh = open(filename, "r")

        results = json.loads(fileh.read())

        project = CiProject.objects.get_or_create(
            name=name,
            branch=branch,
            defaults=dict(
                url=os.path.abspath(os.path.join(settings.DIRNAME, ".."))
            )
        )[0]

        for module, tests in results.items():
            module = TestModule.objects.get_or_create(name=module)[0]

            for test in tests:
                run = TestRun.objects.get_or_create(
                    project=project,
                    module=module,
                    name=test['name'],
                )[0]
                for name, value in dict(
                    status=test['status'],
                    description=test.get("description"),
                    error=test.get("error")
                ).items():
                    setattr(run, name, value)
                run.save()




from socketio_server.utils import send_socketio
import celery
import subprocess


@celery.task
def start_test(name, branch, task_id=None):
    send_socketio("", "test_start", dict(
        name=name,
        branch=branch,
        task_id=task_id,
    ))

    process = subprocess.Popen(["python", "manage.py", "citests",
        name, branch], stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)
    response = process.communicate()

    send_socketio("", "test_end", dict(
        name=name,
        branch=branch,
        response=response,
        task_id=task_id,
    ))



from django.contrib import admin
from ci.models import CiProject, TestModule, TestRun
from danemco.core.widgets import TextAreaNoMce


class TestRunAdmin(admin.ModelAdmin):
    search_fields = ("name", "module")
    list_display = ("name", "project", "module", "status")
    list_display_links = ("name",)
    list_filter = ("project", "module", "status")


class CiProjectAdmin(admin.ModelAdmin):

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == "test_commands":
            kwargs['widget'] = TextAreaNoMce
        print db_field.name
        return admin.ModelAdmin.formfield_for_dbfield(self, db_field, **kwargs)


admin.site.register(CiProject, CiProjectAdmin)
admin.site.register(TestModule)
admin.site.register(TestRun, TestRunAdmin)



from django.test.testcases import TestCase
from ci.models import CiProject


class CiTestCase(TestCase):

    def setUp(self):
        self.ciproject = CiProject.objects.create(
            name="django-ci",
            url="https://freakypie@bitbucket.org/freakypie/django-ci.git",
            branch="master",
            test_commands="""
bin/python django-ci/subtestproject/manage.py test
"""
        )

    def testTest(self):
        self.ciproject.setup_venv()
#         self.ciproject.test()

    def tearDown(self):
        pass  # TODO: delete env



from django.core.management.base import BaseCommand
from optparse import make_option

import daemon
import time
from ci.models import CiProject

class Command(BaseCommand):

    option_list = BaseCommand.option_list + (
        make_option('--chroot_directory', action='store', dest='chroot_directory',
            help='Full path to a directory to set as the effective root directory of \
            the process.'),
        make_option('--working_directory', action='store', dest='working_directory',
            default="/",
            help='Full path of the working directory to which the process should \
            change on daemon start.'),
        make_option('--umask', action='store', dest='umask', default=0, type="int",
            help='File access creation mask ("umask") to set for the process on \
            daemon start.'),
        make_option('--pidfile', action='store', dest='pidfile',
            help='Context manager for a PID lock file. When the daemon context opens \
            and closes, it enters and exits the `pidfile` context manager.'),
        make_option('--detach_process', action='store', dest='detach_process',
            help='If ``True``, detach the process context when opening the daemon \
            context; if ``False``, do not detach.'),
        make_option('--uid', action='store', dest='uid',
            help='The user ID ("UID") value to switch the process to on daemon start.'),
        make_option('--gid', action='store', dest='gid',
            help='The group ID ("GID") value to switch the process to on daemon start.'),
        make_option('--prevent_core', action='store', dest='prevent_core', default=True,
            help='If true, prevents the generation of core files, in order to avoid \
            leaking sensitive information from daemons run as `root`.'),
        make_option('--stdin', action='store', dest='stdin',
            help='Standard In'),
        make_option('--stdout', action='store', dest='stdout',
            help='Standard Out'),
        make_option('--stderr', action='store', dest='stderr',
            help='Standard Error'),
    )
    help = 'Create a daemon'

    chroot_directory = None
    working_directory = '/'
    umask = 0
    detach_process = True
    prevent_core = True
    stdin = None
    stdout = None
    stderr = None
    pidfile = None
    uid = None
    gid = None


    def get_option_value(self, options, name, expected=None):
        value = options.get(name)
        if value == expected:
            value = getattr(self, name)
        # print name, ' ', value
        return value

    def handle(self, name, branch, *args, **options):
        """
        Takes the options and starts a daemon context from them.

        Example::

            python manage.py linkconsumer --pidfile=/var/run/cb_link.pid
                --stdout=/var/log/cb/links.out --stderr=/var/log/cb/links.err

        """
        context = daemon.DaemonContext()

        context.chroot_directory = self.get_option_value(options, 'chroot_directory')
        context.working_directory = self.get_option_value(options, 'working_directory', '/')
        context.umask = self.get_option_value(options, 'umask', 0)
        context.detach_process = self.get_option_value(options, 'detach_process')
        context.prevent_core = self.get_option_value(options, 'prevent_core', True)

        # Get file objects
        stdin = self.get_option_value(options, 'stdin')
        if stdin is not None:
            context.stdin = open(stdin, "r")

        stdout = self.get_option_value(options, 'stdout')
        if stdout is not None:
            context.stdout = open(stdout, "a+")

        stderr = self.get_option_value(options, 'stderr')
        if stderr is not None:
            context.stderr = open(stderr, "a+")

#         # Make pid lock file
#         pidfile = self.get_option_value(options, 'pidfile')
#         if pidfile is not None:
#             context.pidfile = pidlockfile.PIDLockFile(pidfile)

        uid = self.get_option_value(options, 'uid')
        if uid is not None:
            context.uid = uid

        gid = self.get_option_value(options, 'gid')
        if gid is not None:
            context.gid = uid

        context.open()

        self.handle_daemon(name, branch, *args, **options)

    def handle_daemon(self, name, branch, *args, **options):
        """
        Perform the command's actions in the given daemon context
        """
        project = CiProject.objects.get(name=name, branch=branch)
        project.testrun_set.all().delete()
        project.test()



from django.conf.urls import patterns, url, include
from ci.views import TriggerTest
from viewsets import ViewSet

defaultpatterns = patterns(r'^ci/', include("ci.urls"))

urlpatterns = ViewSet.all_urls()

urlpatterns += patterns('',
    url(r'^trigger/(?P<name>[^/]+)/(?P<branch>[\-\_\w]+)/', TriggerTest.as_view()),
)



from django_test_exclude.runners import ExcludeTestSuiteRunner
from django.utils.unittest.runner import TextTestRunner, TextTestResult
from unittest.util import strclass
import traceback
import json


class JSONTestResult(TextTestResult):

    def addSuccess(self, test):
        super(JSONTestResult, self).addSuccess(test)
        self.addResult(test, status="success")

    def addError(self, test, err):
        super(JSONTestResult, self).addError(test, err)
        self.addResult(test, status="error", error=err)

    def addFailure(self, test, err):
        super(JSONTestResult, self).addFailure(test, err)
        self.addResult(test, status="fail", error=err)

    def addSkip(self, test, reason):
        super(JSONTestResult, self).addSkip(test, reason)
        self.addResult(test, status="skip", error=reason)

    def addExpectedFailure(self, test, err):
        super(JSONTestResult, self).addExpectedFailure(test, err)
        self.addResult(test, status="expected failure", error=err)

    def addUnexpectedSuccess(self, test):
        super(JSONTestResult, self).addUnexpectedSuccess(test)
        self.addResult(test, status="unexpected success")

    def addResult(self, test, status, error=None):
        data = dict(
            name="%s" % test._testMethodName,
            status=status,
        )
        desc = test.shortDescription()
        if desc:
            data["description"] = desc
        if error:
            if isinstance(error, basestring):
                data["error"] = error
            else:
                data["error"] = "".join(traceback.format_exception(*error))
            print data['error']
        module = strclass(test.__class__)
        tests = self.results.get(module, [])
        tests.append(data)
        self.results[module] = tests

    def startTestRun(self):
        super(JSONTestResult, self).startTestRun()
        self.results = {}

    def printErrorList(self, flavour, errors):
        for test, err in errors:
            self.stream.writeln(self.separator1)
            self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
            self.stream.writeln(self.separator2)
            self.stream.writeln("%s" % err)


class JSONTestRunner(TextTestRunner):
    resultclass = JSONTestResult


class TestSuiteRunner(ExcludeTestSuiteRunner):

    def run_suite(self, suite, **kwargs):

        self.retval = JSONTestRunner(
            verbosity=self.verbosity, failfast=self.failfast).run(suite)

        return self.retval

    def run_tests(self, test_labels, extra_tests=None, **kwargs):

#         import danemco.utils.traceprints

        test_results = super(TestSuiteRunner, self).run_tests(test_labels, extra_tests, **kwargs)

        fileh = open("testresults.json", "w")
        fileh.write(json.dumps(self.retval.results))
        fileh.close()
        return test_results





from ci.virtualenvapi.manage import VirtualEnvironment
from django.core.management import call_command
from django.db import models
from django.db.models.aggregates import Count
from django.template.defaultfilters import slugify
from os import path
import datetime
import os
import re
import shlex
import signal
import subprocess
import traceback
import git
from celery.result import AsyncResult


class Server:
    ports = range(8003, 8011)
    available_ports = ports[:]
    servers = {}


class CiProject(models.Model):
    class Meta:
        unique_together = ("name", "branch")
    name = models.CharField(max_length=50)
    url = models.CharField(max_length=255)
    branch = models.CharField(max_length=50)
    manage_location = models.CharField(max_length=255, blank=True, null=True)
    settings_location = models.CharField(max_length=255, blank=True, null=True)
    test_commands = models.TextField(null=True, blank=True)
    dependancies = models.ManyToManyField("self", blank=True)
    celery_id = models.CharField(max_length=50)

    def __unicode__(self):
        return u"%s@%s" % (self.name, self.branch)

    @property
    def git_head(self):
        repo = git.Repo(path.join(self.path, self.repo_path))
        return repo.head.commit

    @property
    def testing_status(self):
        if self.celery_id:
            try:
                result = AsyncResult(self.celery_id)
                return result.state
            except:
                traceback.print_exc()
        return ""

    @property
    def path(self):        
        name = "%s@%s" % (slugify(self.name), slugify(self.branch))
        path = 'builds/%s/' % name
        return path

    def get_repo_path(self, url):
        return os.path.basename(url).rsplit(".", 1)[0]

    @property
    def repo_path(self):
        return self.get_repo_path(self.url)
    
    def setup_venv(self):
        path = self.path
        try:
            os.makedirs(path)
        except:
            pass

        print "setting up environment: %s" % path
        env = VirtualEnvironment(path)

        self.pull(env)

        if self.dependancies.exists():
            print "pulling dependancies"
            for dependancy in self.dependancies.all():
                print dependancy
                dependancy.pull(env, self.path)

        requirements_file = os.path.join(env.path, self.repo_path, "requirements.txt")
        if os.path.exists(requirements_file):
            print "setting up requirements"
            with open(requirements_file) as fileh:
                for dependancy in fileh:
                    print dependancy
                    groups = re.match("install ([a-z_\-\d, ]+) from (.+)$", dependancy)
                    if groups:
                        modules = groups.group(1).split(",")
                        location, refid = groups.group(2).rsplit("@", 1)

                        # is there a branch?
                        if ":" not in location:
                            location = location + refid
                            refid = "master"

                        print modules, location, refid

                        dependancy = CiProject(
                            url=location,
                            branch=refid
                        )
                        dependancy.pull(env, self.path)

                        for module in modules:
                            module = module.strip()
                            print module
                            args = [
                                "mklink", "/J",
                                '"%s"' % os.path.join(self.path, "Lib", "site-packages", module),
                                '"%s"' % os.path.join(self.path, dependancy.repo_path, module)
                            ]
                            print " ".join(args)
                            if os.name == "nt":
                                os.system(" ".join(args))
#                                 env._execute(args)
                            else:
                                pass  # not sure right now.
                    else:
                        if os.name == "nt":
                            env._execute([
                                "Scripts/pip", "install", "-q", "--upgrade",
                                dependancy
                            ])
                        else:
                            env._execute([
                                "bin/pip", "install", "-q", "-r",
                                dependancy
                            ])

        return env

    def pull(self, env=None, path=None):
        if not path:
            path = self.path

        print "pulling", self.url, "to", path
        my_path = os.path.join(path, self.repo_path)

        if os.path.exists(my_path):
            repo = git.Repo(my_path)
            created = False
        else:
            repo = git.Repo.clone_from(self.url, my_path)
            print "clone:", repo.clone(self.url)
            created = True

        print repo.git.config(l=True)
        print "checkout:", getattr(repo.heads, self.branch).checkout()
        print "reset:", repo.head.reset(index=True, working_tree=True)
        if not created:
            try:
                print repo.git.pull()
            except:
                print "ERROR"
                traceback.print_exc()
                print

    def test(self):

        env = self.setup_venv()
        envc = os.environ.copy()
        if os.name == "nt":
            envc["PATH"] = str(os.path.abspath(os.path.join(env.path, "Scripts")) + ":" + envc.get("PATH", ""))
        else:
            envc["PATH"] = str(os.path.abspath(os.path.join(env.path, "bin")) + ";" + envc.get("PATH", ""))
        envc["DJANGO_SETTINGS_MODULE"] = str(self.settings_location)
        print envc

        for command in self.test_commands.split("\n"):
            if ";;;" in command:
                command, cwd = command.split(";;;", 1)
            else:
                cwd = None
            cmds = shlex.split(command)
            if cmds:
                env._execute(cmds, cwd=cwd, env=envc)

        call_command("import_testresults", self.name, self.branch,
            os.path.join(env.path, self.repo_path, "superiordjango", "testresults.json"))

    def stats(self):
        return self.testrun_set.values("status").annotate(count=Count("status"))

    def _create_server(self):
        if len(Server.available_ports) > 0:
            port = Server.available_ports[0]
            Server.available_ports = Server.available_ports[1:]
            
            env = self.setup_venv()
            args = ["python", "manage.py", "runserver", "0.0.0.0:%s" % port]
            cwd = os.path.join(env.path, os.path.dirname(self.manage_location))
            envc = os.environ.copy()
            envc["PATH"] = str(os.path.abspath(os.path.join(env.path, "Scripts")) + ":" + envc.get("PATH", ""))
            envc["PYTHONPATH"] = ".;%s" % self.repo_path
            envc["DJANGO_SETTINGS_MODULE"] = str(self.settings_location)
            print envc
            server = subprocess.Popen(args, env=envc, cwd=cwd,
                preexec_fn=os.setsid)
            Server.servers[self] = {"port": port, "process": server}
        return self.get_server()

    def _stop_server(self):
        server = Server.servers.get(self)
        if server:

            os.killpg(server['process'].pid, signal.SIGKILL)

            Server.available_ports.append(server['port'])
            del Server.servers[self]

    def get_server(self):
        if self in Server.servers:
            server = Server.servers.get(self)
            if not server['process'].poll() is None:
                Server.available_ports.append(server['port'])
                del Server.servers[self]
                
        return Server.servers.get(self)

    def get_server_url(self, request):
        server = self.server()
        if server:
            return "http://%s:%s" % (request.get_host(), server[0])

    @property
    def last_run(self):
        return self.testrun_set.latest("created_at")


class TestModule(models.Model):
    class Meta:
        ordering = ("name",)
    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name


class TestRunManager(models.Manager):

    def failures(self):
        return self.get_query_set().filter(status__in=("fail", "error", "unexpected success"))

    def successes(self):
        return self.get_query_set().exclude(status__in=("fail", "error", "unexpected success"))


class TestRun(models.Model):
    class Meta:
        ordering = ("module", "name",)
    created_at = models.DateTimeField(auto_now=True)
    project = models.ForeignKey(CiProject)
    module = models.ForeignKey(TestModule)
    name = models.CharField(max_length=255)
    status = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    error = models.TextField(blank=True, null=True)

    objects = TestRunManager()

    def __unicode__(self):
        return self.name

    def label(self):
        return {
            "success": "success",
            "skip": "default",
            "fail": "important",
            "error": "important",
            "expected failure": "success",
            "unexpected success": "important",
        }.get(self.status, "important")
        



from ci.models import CiProject
from ci.tasks import start_test
from django.contrib import messages
from django.core.urlresolvers import reverse_lazy
from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.views.generic.base import View
from django.views.generic.detail import DetailView, SingleObjectMixin
from pprint import pprint
import json
import subprocess
from viewsets import ViewSet, ViewSetMixin
from celery.result import AsyncResult
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt


class BaseViewSet(ViewSet):    

    def protect(self, function):
        if not hasattr(function, "protected"):
            def inner(self, request, *args, **kwargs):
#                 if not request.user.is_authenticated():
#                     return redirect("%s?next=%s" % (settings.LOGIN_URL, request.path))
                return function(self, request, *args, **kwargs)
            inner.protected = True
            return inner
        return function

ci_viewset = BaseViewSet(model=CiProject, template_dir="ci/ci-projects")


@ci_viewset.instance_view("test")
class TestDetailView(ViewSetMixin, DetailView):
    
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return DetailView.dispatch(self, request, *args, **kwargs)
    dispatch.csrf_exempt = True

    def post(self, request, pk):
        self.object = self.get_object()
        
        if self.object.celery_id:
            result = AsyncResult(self.object.celery_id)
            result.revoke(terminate=True)

        result = start_test.apply_async([self.object.name, self.object.branch])
        self.object.celery_id = result.task_id
        self.object.save()

        messages.info(request, "test queued")

        return redirect(request.META.get("HTTP_REFERER",
            reverse_lazy("ci-projects:detail", args=[self.object.pk])))


@ci_viewset.instance_view("pull")
class PullProjectView(ViewSetMixin, SingleObjectMixin, View):

    def post(self, request, pk):
        self.object = self.get_object()
        self.object.pull(None)
        messages.success(request, "Pulled")
        return redirect(request.META.get("HTTP_REFERER",
            reverse_lazy("ci-projects:detail", args=[self.object.pk])))


@ci_viewset.instance_view("start")
class StartServerView(ViewSetMixin, SingleObjectMixin, View):

    def post(self, request, pk):
        self.object = self.get_object()
        server = self.object.get_server()
        if not server:
            server = self.object._create_server()
            if not server:
                messages.error(request, "Sorry, couldn't create server")
        return redirect(request.META.get("HTTP_REFERER",
            reverse_lazy("ci-projects:detail", args=[self.object.pk])))


@ci_viewset.instance_view("stop")
class StopServerView(ViewSetMixin, SingleObjectMixin, View):

    def post(self, request, pk):
        self.object = self.get_object()
        self.object._stop_server()
        messages.success(request, "Server stopped")
        return redirect(request.META.get("HTTP_REFERER",
            reverse_lazy("ci-projects:detail", args=[self.object.pk])))


class TriggerTest(View):

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(TriggerTest, self).dispatch(request, *args, **kwargs)

    def get(self, request, name, branch):
        print request.GET

        return HttpResponse("OK")

    def post(self, request, name, branch):
        pprint(json.loads(request.body))

        process = subprocess.Popen(["python", "manage.py", "citests", name, branch])
        print process.communicate()

        return HttpResponse("OK")



from ci.models import CiProject
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, name, branch, *args, **options):
        project = CiProject.objects.get(name=name, branch=branch)
        project.testrun_set.all().delete()
        project.test()



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CiProject'
        db.create_table('ci_ciproject', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('url', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('branch', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('test_commands', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('ci', ['CiProject'])

        # Adding unique constraint on 'CiProject', fields ['name', 'branch']
        db.create_unique('ci_ciproject', ['name', 'branch'])

        # Adding M2M table for field dependancies on 'CiProject'
        db.create_table('ci_ciproject_dependancies', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('from_ciproject', models.ForeignKey(orm['ci.ciproject'], null=False)),
            ('to_ciproject', models.ForeignKey(orm['ci.ciproject'], null=False))
        ))
        db.create_unique('ci_ciproject_dependancies', ['from_ciproject_id', 'to_ciproject_id'])

        # Adding model 'TestModule'
        db.create_table('ci_testmodule', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('ci', ['TestModule'])

        # Adding model 'TestRun'
        db.create_table('ci_testrun', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('project', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ci.CiProject'])),
            ('module', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ci.TestModule'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('status', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('error', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('ci', ['TestRun'])


    def backwards(self, orm):
        # Removing unique constraint on 'CiProject', fields ['name', 'branch']
        db.delete_unique('ci_ciproject', ['name', 'branch'])

        # Deleting model 'CiProject'
        db.delete_table('ci_ciproject')

        # Removing M2M table for field dependancies on 'CiProject'
        db.delete_table('ci_ciproject_dependancies')

        # Deleting model 'TestModule'
        db.delete_table('ci_testmodule')

        # Deleting model 'TestRun'
        db.delete_table('ci_testrun')


    models = {
        'ci.ciproject': {
            'Meta': {'unique_together': "(('name', 'branch'),)", 'object_name': 'CiProject'},
            'branch': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'dependancies': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'dependancies_rel_+'", 'blank': 'True', 'to': "orm['ci.CiProject']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'test_commands': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'ci.testmodule': {
            'Meta': {'ordering': "('name',)", 'object_name': 'TestModule'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'ci.testrun': {
            'Meta': {'ordering': "('module', 'name')", 'object_name': 'TestRun'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'error': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'module': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ci.TestModule']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ci.CiProject']"}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['ci']


from os import linesep
import os.path
import re
import subprocess

from ci.virtualenvapi.util import split_package_name

class VirtualEnvironment(object):
    # True if the virtual environment has been set up through open_or_create()
    _ready = False

    def __init__(self, path):
        # remove trailing slash so os.path.split() behaves correctly
        if path[-1] == '/':
            path = path[:-1]
        self.path = path

    def __str__(self):
        return self.path

    @property
    def _pip_rpath(self):
        """The relative path (from environment root) to pip."""
        return os.path.join('bin', 'pip')

    @property
    def root(self):
        """The root directory that this virtual environment exists in."""
        return os.path.split(self.path)[0]

    @property
    def name(self):
        """The name of this virtual environment (taken from its path)."""
        return os.path.basename(self.path)

    @property
    def _logfile(self):
        """Absolute path of the log file for recording installation output."""
        return os.path.join(self.path, 'build.log')

    def _create(self):
        """Executes `virtualenv` to create a new environment."""
        out = subprocess.check_output(['virtualenv', "--system-site-packages", self.name], cwd=self.root)
        self._write_to_log(out, truncate=True) # new log

    def _execute(self, args, cwd=None, env=None):
        """Executes the given command inside the environment and returns the output."""
        if not self._ready:
            self.open_or_create()
        try:
#             output = subprocess.check_output(args, cwd=self.path)

            if cwd is None:
                cwd = self.path
            else:
                cwd = os.path.join(self.path, cwd)

            process = subprocess.Popen(args, stdout=subprocess.PIPE, cwd=cwd, env=env)
            out, err = process.communicate()

            if out and out.strip():
                print "STDOUT:", out.strip()
            if err and err.strip():
                print "STDERR:", err.strip()
            return out
        except OSError, e:
            # raise a more meaningful error with the program name
            prog = args[0]
            raise OSError('%s: %s' % (prog, str(e)))

    def _write_to_log(self, s, truncate=False):
        """Writes the given output to the log file, appending unless `truncate` is True."""
        # if truncate is True, set write mode to truncate
        choice = { True: 'w',
                   False: 'a'
                 }
        with open(self._logfile, choice[truncate]) as fp:
            fp.write(s + linesep)

    def _pip_exists(self):
        """Returns True if pip exists inside the virtual environment. Can be
        used as a naive way to verify that the envrionment is installed."""
        return os.path.isfile(os.path.join(self.path, self._pip_rpath))

    def open_or_create(self):
        """Attempts to open the virtual environment or creates it if it
        doesn't exist.
        XXX this should probably be expanded to do some proper checking?"""
        if not self._pip_exists():
            self._create()
        self._ready = True

    def install(self, package, force=False):
        """Installs the given package (given in pip's package syntax) 
        into this virtual environment only if it is not already installed.
        If `force` is True, force an installation."""
        if not force and self.is_installed(package):
            self._write_to_log('%s is already installed, skipping (use force=True to override)' % package)
            return
        out = self._execute([self._pip_rpath, 'install', package])
        self._write_to_log(out)

    def uninstall(self, package):
        """Uninstalls the given package (given in pip's package syntax) from
        this virtual environment."""
        if not self.is_installed(package):
            self._write_to_log('%s is not installed, skipping')
            return
        out = self._execute([self._pip_rpath, 'uninstall', '-y', package])
        self._write_to_log(out)

    def is_installed(self, package):
        """Returns True if the given package (given in pip's package syntax)
        is installed in the virtual environment."""
        if package.endswith('.git'):
            pkg_name = os.path.split(package)[1][:-4]
            return pkg_name in self.installed_package_names
        pkg_tuple = split_package_name(package)
        if pkg_tuple[1] is not None:
            return pkg_tuple in self.installed_packages
        else:
            return pkg_tuple[0] in self.installed_package_names

    def upgrade(self, package):
        """Shortcut method to upgrade a package by forcing a reinstall.
        Note that this may not actually upgrade but merely reinstall if there
        is no newer version to install."""
        self.install(package, force=True)

    @property
    def installed_packages(self):
        """List of all packages that are installed in this environment."""
        pkgs = [] #: [(name, ver), ..]
        l = self._execute([self._pip_rpath, 'freeze', '-l']).split(linesep)
        for p in l:
            if p == '': continue
            pkgs.append(split_package_name(p))
        return pkgs

    @property
    def installed_package_names(self):
        """List of all package names that are installed in this environment."""
        return [name.lower() for name, _ in self.installed_packages]



from setuptools import setup, find_packages

setup(
    name='django-ci',
    version='0.0.4',
    author='John Leith',
    author_email='leith.john@gmail.com',
    packages=find_packages(),
    url='http://pypi.python.org/pypi/django-ci/',
    download_url='https://bitbucket.org/freakypie/django-ci',
    description='Django continuous integration and testing',
    long_description=open('README.rst').read(),
    install_requires=[
        "Django==1.4.5",
        "django-viewgroups>=0.0.4",
    	"socketio_server>=0.0.5",
    	"kombu",
    	"django-celery"
    ],
    package_data = {
        '': ['*.txt', '*.rst', '*.md', '*.html'],
    },
)



from django.template.base import Library
import datetime
from django.template.defaultfilters import date


register = Library()

@register.filter
def host_only(request):
    return request.get_host().rsplit(":", 1)[0]


@register.filter
def timestamp(commit_time, format=None):
    date = datetime.datetime.fromtimestamp(commit_time)
    if format:
        return date(date, format)
    return date



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'CiProject.manage_location'
        db.add_column('ci_ciproject', 'manage_location',
                      self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True),
                      keep_default=False)

        # Adding field 'CiProject.settings_location'
        db.add_column('ci_ciproject', 'settings_location',
                      self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'CiProject.manage_location'
        db.delete_column('ci_ciproject', 'manage_location')

        # Deleting field 'CiProject.settings_location'
        db.delete_column('ci_ciproject', 'settings_location')


    models = {
        'ci.ciproject': {
            'Meta': {'unique_together': "(('name', 'branch'),)", 'object_name': 'CiProject'},
            'branch': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'dependancies': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'dependancies_rel_+'", 'blank': 'True', 'to': "orm['ci.CiProject']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'manage_location': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'settings_location': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'test_commands': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'ci.testmodule': {
            'Meta': {'ordering': "('name',)", 'object_name': 'TestModule'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'ci.testrun': {
            'Meta': {'ordering': "('module', 'name')", 'object_name': 'TestRun'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'error': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'module': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ci.TestModule']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ci.CiProject']"}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['ci']



def split_package_name(p):
    """Splits the given package name and returns a tuple (name, ver)."""
    s = p.split('==')
    if len(s) == 1:
        return (s[0], None)
    else:
        return (s[0], s[1])
